@@ -64,11 +64,11 @@
 #include <misc/portable.h>
 #include <reentrant.h>
 #include <rpc/rpc.h>
+#include <rpc/svc_rqst.h>
 #include "rpc_com.h"
 #include "clnt_internal.h"
-#include "rpc_dplx_internal.h"
+#include "svc_internal.h"
 #include "rpc_ctx.h"
-#include <rpc/svc_rqst.h>
 
 #define MAX_DEFAULT_FDS                 20000
 
@@ -84,8 +84,6 @@ static void clnt_dg_abort(CLIENT *);
 static bool clnt_dg_control(CLIENT *, u_int, void *);
 static void clnt_dg_destroy(CLIENT *);
 
-static const char mem_err_clnt_dg[] = "clnt_dg_create: out of memory";
-
 /*
  * Connection less client creation returns with client handle parameters.
  * Default options are set, which the user can change using clnt_control().
@@ -99,20 +97,21 @@ static const char mem_err_clnt_dg[] = "clnt_dg_create: out of memory";
  * If svcaddr is NULL, returns NULL.
  */
 CLIENT *
-clnt_dg_ncreate(int fd,	/* open file descriptor */
-		const struct netbuf *svcaddr,	/* servers address */
-		rpcprog_t program,	/* program number */
-		rpcvers_t version,	/* version number */
-		u_int sendsz,	/* buffer recv size */
-		u_int recvsz /* buffer send size */)
+clnt_dg_ncreatef(const int fd,	/* open file descriptor */
+		 const struct netbuf *svcaddr,	/* servers address */
+		 const rpcprog_t program,	/* program number */
+		 const rpcvers_t version,	/* version number */
+		 const u_int sendsz,	/* buffer recv size */
+		 const u_int recvsz,	/* buffer send size */
+		 const uint32_t flags)
 {
-	CLIENT *clnt = NULL;	/* client handle */
-	struct cx_data *cx = NULL;	/* private data */
-	struct cu_data *cu = NULL;
+	CLIENT *clnt;		/* client handle */
+	struct cx_data *cx;
+	struct cu_data *cu;
+	SVCXPRT *xprt;
+	struct svc_dg_xprt *su;
 	struct timespec now;
 	struct rpc_msg call_msg;
-	struct __rpc_sockinfo si;
-	uint32_t oflags;
 	int one = 1;
 
 	if (svcaddr == NULL) {
@@ -120,57 +119,54 @@ clnt_dg_ncreate(int fd,	/* open file descriptor */
 		return (NULL);
 	}
 
-	if (!__rpc_fd2sockinfo(fd, &si)) {
-		rpc_createerr.cf_stat = RPC_TLIERROR;
-		rpc_createerr.cf_error.re_errno = 0;
-		return (NULL);
-	}
-	/*
-	 * Find the receive and the send size
-	 */
-	sendsz = __rpc_get_t_size(si.si_af, si.si_proto, (int)sendsz);
-	recvsz = __rpc_get_t_size(si.si_af, si.si_proto, (int)recvsz);
-	if ((sendsz == 0) || (recvsz == 0)) {
+	/* find or create shared fd state; ref+1 */
+	xprt = svc_dg_ncreatef(fd, sendsz, recvsz, flags);
+	if (!xprt) {
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d svc_dg_ncreatef failed",
+			__func__, fd);
 		rpc_createerr.cf_stat = RPC_TLIERROR;	/* XXX */
 		rpc_createerr.cf_error.re_errno = 0;
 		return (NULL);
 	}
+	su = su_data(xprt);
 
-	clnt = mem_alloc(sizeof(CLIENT));
-
-	mutex_init(&clnt->cl_lock, NULL);
-	clnt->cl_flags = CLNT_FLAG_NONE;
-
-	/*
-	 * Should be multiple of 4 for XDR.
-	 */
-	sendsz = ((sendsz + 3) / 4) * 4;
-	recvsz = ((recvsz + 3) / 4) * 4;
-	cx = alloc_cx_data(CX_DG_DATA, sendsz, recvsz);
-	if (cx == NULL)
-		goto err1;
+	/* buffer sizes should match svc side */
+	cx = alloc_cx_data(CX_DG_DATA, su->su_sendsz, su->su_recvsz);
+	cx->cx_rec = &su->su_dr;
 	cu = CU_DATA(cx);
+
 	(void)memcpy(&cu->cu_raddr, svcaddr->buf, (size_t) svcaddr->len);
 	cu->cu_rlen = svcaddr->len;
+
 	/* Other values can also be set through clnt_control() */
 	cu->cu_wait.tv_sec = 15;	/* heuristically chosen */
 	cu->cu_wait.tv_usec = 0;
 	cu->cu_total.tv_sec = -1;
 	cu->cu_total.tv_usec = -1;
-	cu->cu_sendsz = sendsz;
-	cu->cu_recvsz = recvsz;
 	cu->cu_async = false;
 	cu->cu_connect = false;
 	cu->cu_connected = false;
+
+	/*
+	 * initialize call message
+	 */
 	(void)clock_gettime(CLOCK_MONOTONIC_FAST, &now);
 	call_msg.rm_xid = __RPC_GETXID(&now);	/* XXX? */
-	call_msg.rm_call.cb_prog = program;
-	call_msg.rm_call.cb_vers = version;
-	xdrmem_create(&(cu->cu_outxdrs), cu->cu_outbuf, sendsz, XDR_ENCODE);
+	call_msg.cb_prog = program;
+	call_msg.cb_vers = version;
+
+	xdrmem_create(&(cu->cu_outxdrs), cu->cu_outbuf, su->su_sendsz,
+		      XDR_ENCODE);
 	if (!xdr_callhdr(&(cu->cu_outxdrs), &call_msg)) {
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d xdr_callhdr failed",
+			__func__, fd);
 		rpc_createerr.cf_stat = RPC_CANTENCODEARGS;	/* XXX */
 		rpc_createerr.cf_error.re_errno = 0;
-		goto err2;
+		SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
+		free_cx_data(cx);
+		return (NULL);
 	}
 	cu->cu_xdrpos = XDR_GETPOS(&(cu->cu_outxdrs));
 
@@ -185,30 +181,14 @@ clnt_dg_ncreate(int fd,	/* open file descriptor */
 	}
 #endif
 	ioctl(fd, FIONBIO, (char *)(void *)&one);
-	/*
-	 * By default, closeit is always false. It is users responsibility
-	 * to do a close on it, else the user may use clnt_control
-	 * to let clnt_destroy do it for him/her.
-	 */
-	cu->cu_closeit = false;
-	cu->cu_fd = fd;
-	clnt->cl_ops = clnt_dg_ops();
-	clnt->cl_p1 = cx;
-	clnt->cl_p2 = rpc_dplx_lookup_rec(fd, RPC_DPLX_LKP_FLAG_NONE,
-					  &oflags); /* ref+1 */
-	clnt->cl_tp = NULL;
-	clnt->cl_netid = NULL;
 
+	clnt = &cx->cx_c;
+	clnt->cl_ops = clnt_dg_ops();
+
+	__warnx(TIRPC_DEBUG_FLAG_CLNT_DG,
+		"%s: fd %d completed",
+		__func__, fd);
 	return (clnt);
- err1:
-	__warnx(TIRPC_DEBUG_FLAG_CLNT_DG, mem_err_clnt_dg);
-	rpc_createerr.cf_stat = RPC_SYSTEMERROR;
-	rpc_createerr.cf_error.re_errno = errno;
- err2:
-	mem_free(clnt, sizeof(CLIENT));
-	if (cx)
-		free_cx_data(cx);
-	return (NULL);
 }
 
 static enum clnt_stat
@@ -222,27 +202,32 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 	     struct timeval utimeout
 	     /* seconds to wait before giving up */)
 {
-	struct cu_data *cu = CU_DATA((struct cx_data *)clnt->cl_p1);
+	struct cx_data *cx = CX_DATA(clnt);
+	struct cu_data *cu = CU_DATA(cx);
+	struct rpc_dplx_rec *rec = cx->cx_rec;
+	SVCXPRT *xprt = &rec->xprt;
 	XDR *xdrs;
-	size_t outlen = 0;
+	struct sockaddr *sa;
 	struct rpc_msg reply_msg;
 	XDR reply_xdrs;
-	bool ok;
-	int nrefreshes = 2;	/* number of times to refresh cred */
 	struct timeval timeout;
 	struct pollfd fd;
 	int total_time, nextsend_time, tv = 0;
-	struct sockaddr *sa;
 	socklen_t __attribute__ ((unused)) inlen, salen;
+	size_t outlen = 0;
 	ssize_t recvlen = 0;
+	int nrefreshes = 2;	/* number of times to refresh cred */
+	int xp_fd = xprt->xp_fd;
 	u_int32_t xid, inval, outval;
-	bool slocked = false;
+	bool ok;
+	bool slocked = true;
 	bool rlocked = false;
 	bool once = true;
 
-	outlen = 0;
-	rpc_dplx_slc(clnt);
-	slocked = true;
+	/* Need lock for cu.
+	 */
+	mutex_lock(&clnt->cl_lock);
+
 	if (cu->cu_total.tv_usec == -1)
 		timeout = utimeout;	/* use supplied timeout */
 	else
@@ -252,10 +237,10 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 
 	if (cu->cu_connect && !cu->cu_connected) {
 		if (connect
-		    (cu->cu_fd, (struct sockaddr *)&cu->cu_raddr,
+		    (xp_fd, (struct sockaddr *)&cu->cu_raddr,
 		     cu->cu_rlen) < 0) {
-			cu->cu_error.re_errno = errno;
-			cu->cu_error.re_status = RPC_CANTSEND;
+			cx->cx_error.re_errno = errno;
+			cx->cx_error.re_status = RPC_CANTSEND;
 			goto out;
 		}
 		cu->cu_connected = 1;
@@ -271,12 +256,15 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 	/* Clean up in case the last call ended in a longjmp(3) call. */
  call_again:
 	if (!slocked) {
-		rpc_dplx_slc(clnt);
 		slocked = true;
+		mutex_lock(&clnt->cl_lock);
 	}
 	xdrs = &(cu->cu_outxdrs);
-	if (cu->cu_async == true && xargs == NULL)
+	if (cu->cu_async == true && xargs == NULL) {
+		mutex_unlock(&clnt->cl_lock);
+		slocked = false;
 		goto get_reply;
+	}
 	xdrs->x_op = XDR_ENCODE;
 	XDR_SETPOS(xdrs, cu->cu_xdrpos);
 	/*
@@ -291,16 +279,18 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 	if ((!XDR_PUTINT32(xdrs, (int32_t *) &proc))
 	    || (!AUTH_MARSHALL(auth, xdrs))
 	    || (!AUTH_WRAP(auth, xdrs, xargs, argsp))) {
-		cu->cu_error.re_status = RPC_CANTENCODEARGS;
+		cx->cx_error.re_status = RPC_CANTENCODEARGS;
 		goto out;
 	}
 	outlen = (size_t) XDR_GETPOS(xdrs);
+	mutex_unlock(&clnt->cl_lock);
+	slocked = false;
 
  send_again:
 	nextsend_time = cu->cu_wait.tv_sec * 1000 + cu->cu_wait.tv_usec / 1000;
-	if (sendto(cu->cu_fd, cu->cu_outbuf, outlen, 0, sa, salen) != outlen) {
-		cu->cu_error.re_errno = errno;
-		cu->cu_error.re_status = RPC_CANTSEND;
+	if (sendto(xp_fd, cu->cu_outbuf, outlen, 0, sa, salen) != outlen) {
+		cx->cx_error.re_errno = errno;
+		cx->cx_error.re_status = RPC_CANTSEND;
 		goto out;
 	}
 
@@ -310,17 +300,18 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 	 * some clock time to spare while the packets are in flight.
 	 * (We assume that this is actually only executed once.)
 	 */
-	rpc_dplx_suc(clnt);
-	slocked = false;
-
-	rpc_dplx_rlc(clnt);
+	rpc_dplx_rli(rec);
 	rlocked = true;
 
-	reply_msg.acpted_rply.ar_verf = _null_auth;
-	reply_msg.acpted_rply.ar_results.where = NULL;
-	reply_msg.acpted_rply.ar_results.proc = (xdrproc_t) xdr_void;
+	if (!xprt->xp_ev)
+		svc_rqst_evchan_reg(__svc_params->ev_u.evchan.id, xprt,
+				    SVC_RQST_FLAG_CHAN_AFFINITY);
 
-	fd.fd = cu->cu_fd;
+	reply_msg.RPCM_ack.ar_verf = _null_auth;
+	reply_msg.RPCM_ack.ar_results.where = NULL;
+	reply_msg.RPCM_ack.ar_results.proc = (xdrproc_t) xdr_void;
+
+	fd.fd = xp_fd;
 	fd.events = POLLIN;
 	fd.revents = 0;
 	while ((total_time > 0) || once) {
@@ -329,18 +320,18 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 		switch (poll(&fd, 1, tv)) {
 		case 0:
 			total_time -= tv;
-			rpc_dplx_ruc(clnt);
+			rpc_dplx_rui(rec);
 			rlocked = false;
 			if (total_time <= 0) {
-				cu->cu_error.re_status = RPC_TIMEDOUT;
+				cx->cx_error.re_status = RPC_TIMEDOUT;
 				goto out;
 			}
 			goto send_again;
 		case -1:
 			if (errno == EINTR)
 				continue;
-			cu->cu_error.re_status = RPC_CANTRECV;
-			cu->cu_error.re_errno = errno;
+			cx->cx_error.re_status = RPC_CANTRECV;
+			cx->cx_error.re_errno = errno;
 			goto out;
 		}
 		break;
@@ -365,7 +356,7 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 		msg.msg_flags = 0;
 		msg.msg_control = cbuf;
 		msg.msg_controllen = 256;
-		ret = recvmsg(cu->cu_fd, &msg, MSG_ERRQUEUE);
+		ret = recvmsg(xp_fd, &msg, MSG_ERRQUEUE);
 		if (ret >= 0 && memcmp(cbuf + 256, cu->cu_outbuf, ret) == 0
 		    && (msg.msg_flags & MSG_ERRQUEUE)
 		    && ((msg.msg_namelen == 0 && ret >= 12)
@@ -380,8 +371,8 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 				    && (cmsg->cmsg_type == IP_RECVERR)) {
 					e = (struct sock_extended_err *)
 					    CMSG_DATA(cmsg);
-					cu->cu_error.re_errno = e->ee_errno;
-					cu->cu_error.re_status = RPC_CANTRECV;
+					cx->cx_error.re_errno = e->ee_errno;
+					cx->cx_error.re_status = RPC_CANTRECV;
 				}
 	}
 #endif
@@ -389,18 +380,18 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 	/* We have some data now */
 	do {
 		recvlen =
-		    recvfrom(cu->cu_fd, cu->cu_inbuf, cu->cu_recvsz, 0, NULL,
+		    recvfrom(xp_fd, cu->cu_inbuf, cu->cu_recvsz, 0, NULL,
 			     NULL);
 	} while (recvlen < 0 && errno == EINTR);
 	if (recvlen < 0 && errno != EWOULDBLOCK) {
-		cu->cu_error.re_errno = errno;
-		cu->cu_error.re_status = RPC_CANTRECV;
+		cx->cx_error.re_errno = errno;
+		cx->cx_error.re_status = RPC_CANTRECV;
 		goto out;
 	}
 
 	if (recvlen < sizeof(u_int32_t)) {
 		total_time -= tv;
-		rpc_dplx_ruc(clnt);
+		rpc_dplx_rui(rec);
 		rlocked = false;
 		goto send_again;
 	}
@@ -412,7 +403,7 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 		memcpy(&outval, cu->cu_outbuf, sizeof(u_int32_t));
 		if (inval != outval) {
 			total_time -= tv;
-			rpc_dplx_ruc(clnt);
+			rpc_dplx_rui(rec);
 			rlocked = false;
 			goto send_again;
 		}
@@ -428,97 +419,74 @@ clnt_dg_call(CLIENT *clnt,	/* client handle */
 	/* XDR_DESTROY(&reply_xdrs); save a few cycles on noop destroy */
 	if (ok) {
 		if ((reply_msg.rm_reply.rp_stat == MSG_ACCEPTED)
-		    && (reply_msg.acpted_rply.ar_stat == SUCCESS))
-			cu->cu_error.re_status = RPC_SUCCESS;
+		    && (reply_msg.RPCM_ack.ar_stat == SUCCESS))
+			cx->cx_error.re_status = RPC_SUCCESS;
 		else
-			_seterr_reply(&reply_msg, &(cu->cu_error));
+			_seterr_reply(&reply_msg, &(cx->cx_error));
 
-		if (cu->cu_error.re_status == RPC_SUCCESS) {
+		if (cx->cx_error.re_status == RPC_SUCCESS) {
 			if (!AUTH_VALIDATE
-			    (auth, &reply_msg.acpted_rply.ar_verf)) {
-				cu->cu_error.re_status = RPC_AUTHERROR;
-				cu->cu_error.re_why = AUTH_INVALIDRESP;
+			    (auth, &reply_msg.RPCM_ack.ar_verf)) {
+				cx->cx_error.re_status = RPC_AUTHERROR;
+				cx->cx_error.re_why = AUTH_INVALIDRESP;
 			} else
 			    if (!AUTH_UNWRAP
 				(auth, &reply_xdrs, xresults, resultsp)) {
-				if (cu->cu_error.re_status == RPC_SUCCESS)
-					cu->cu_error.re_status =
+				if (cx->cx_error.re_status == RPC_SUCCESS)
+					cx->cx_error.re_status =
 					    RPC_CANTDECODERES;
 			}
-			if (reply_msg.acpted_rply.ar_verf.oa_base != NULL) {
-				xdrs->x_op = XDR_FREE;
-				(void)xdr_opaque_auth(xdrs,
-						      &(reply_msg.acpted_rply.
-							ar_verf));
-			}
 		}
 		/* end successful completion */
 		/*
 		 * If unsuccesful AND error is an authentication error
 		 * then refresh credentials and try again, else break
 		 */
-		else if (cu->cu_error.re_status == RPC_AUTHERROR)
+		else if (cx->cx_error.re_status == RPC_AUTHERROR)
 			/* maybe our credentials need to be refreshed ... */
 			if (nrefreshes > 0 && AUTH_REFRESH(auth, &reply_msg)) {
 				nrefreshes--;
-				rpc_dplx_ruc(clnt);
+				rpc_dplx_rui(rec);
 				rlocked = false;
 				goto call_again;
 			}
 		/* end of unsuccessful completion */
 	} /* end of valid reply message */
 	else
-		cu->cu_error.re_status = RPC_CANTDECODERES;
+		cx->cx_error.re_status = RPC_CANTDECODERES;
 
 out:
 	if (slocked)
-		rpc_dplx_suc(clnt);
+		mutex_unlock(&clnt->cl_lock);
 	if (rlocked)
-		rpc_dplx_ruc(clnt);
+		rpc_dplx_rui(rec);
 
-	return (cu->cu_error.re_status);
+	return (cx->cx_error.re_status);
 }
 
 static void
 clnt_dg_geterr(CLIENT *clnt, struct rpc_err *errp)
 {
-	struct cu_data *cu = CU_DATA((struct cx_data *)clnt->cl_p1);
-	*errp = cu->cu_error;
+	struct cx_data *cx = CX_DATA(clnt);
+
+	*errp = cx->cx_error;
 }
 
 static bool
 clnt_dg_freeres(CLIENT *clnt, xdrproc_t xdr_res, void *res_ptr)
 {
-	struct cu_data *cu = CU_DATA((struct cx_data *)clnt->cl_p1);
+	struct cx_data *cx = CX_DATA(clnt);
+	struct cu_data *cu = CU_DATA(cx);
 	XDR *xdrs;
-	sigset_t mask, newmask;
-	bool dummy = 0;
 
 	/* XXX guard against illegal invocation from libc (will fix) */
 	if (!xdr_res)
-		goto out;
+		return (0);
 
 	xdrs = &(cu->cu_outxdrs);	/* XXX outxdrs? */
-
-	/* Handle our own signal mask here, the signal section is
-	 * larger than the wait (not 100% clear why) */
-	sigfillset(&newmask);
-	thr_sigsetmask(SIG_SETMASK, &newmask, &mask);
-
-	/* barrier recv channel */
-	rpc_dplx_rwc(clnt, rpc_flag_clear);
-
 	xdrs->x_op = XDR_FREE;
-	if (xdr_res)
-		dummy = (*xdr_res) (xdrs, res_ptr);
 
-	thr_sigsetmask(SIG_SETMASK, &mask, NULL);
-
-	/* signal recv channel */
-	rpc_dplx_rsc(clnt, RPC_DPLX_FLAG_NONE);
-
- out:
-	return (dummy);
+	return ((*xdr_res)(xdrs, res_ptr));
 }
 
 static bool
@@ -541,21 +509,29 @@ clnt_dg_abort(CLIENT *h)
 static bool
 clnt_dg_control(CLIENT *clnt, u_int request, void *info)
 {
-	struct cu_data *cu = CU_DATA((struct cx_data *)clnt->cl_p1);
+	struct cx_data *cx = CX_DATA(clnt);
+	struct cu_data *cu = CU_DATA(cx);
+	struct rpc_dplx_rec *rec = cx->cx_rec;
 	struct netbuf *addr;
 	bool rslt = true;
 
 	/* always take recv lock first, if taking both locks together */
-	rpc_dplx_rlc(clnt);
-	rpc_dplx_slc(clnt);
+	rpc_dplx_rli(rec);
+	mutex_lock(&clnt->cl_lock);
 
+	/*
+	 * By default, SVC_XPRT_FLAG_CLOSE starts false.  It is user
+	 * responsibility to do a close on the fd, or set by cl_control.
+	 */
 	switch (request) {
 	case CLSET_FD_CLOSE:
-		cu->cu_closeit = true;
+		(void)atomic_set_uint16_t_bits(&rec->xprt.xp_flags,
+						SVC_XPRT_FLAG_CLOSE);
 		rslt = true;
 		goto unlock;
 	case CLSET_FD_NCLOSE:
-		cu->cu_closeit = false;
+		(void)atomic_clear_uint16_t_bits(&rec->xprt.xp_flags,
+						SVC_XPRT_FLAG_CLOSE);
 		rslt = true;
 		goto unlock;
 	}
@@ -591,7 +567,7 @@ clnt_dg_control(CLIENT *clnt, u_int request, void *info)
 		*(struct timeval *)info = cu->cu_wait;
 		break;
 	case CLGET_FD:
-		*(int *)info = cu->cu_fd;
+		*(int *)info = rec->xprt.xp_fd;
 		break;
 	case CLGET_SVC_ADDR:
 		addr = (struct netbuf *)info;
@@ -670,8 +646,8 @@ clnt_dg_control(CLIENT *clnt, u_int request, void *info)
 	}
 
  unlock:
-	rpc_dplx_ruc(clnt);
-	rpc_dplx_suc(clnt);
+	rpc_dplx_rui(rec);
+	mutex_unlock(&clnt->cl_lock);
 
 	return (rslt);
 }
@@ -679,38 +655,15 @@ clnt_dg_control(CLIENT *clnt, u_int request, void *info)
 static void
 clnt_dg_destroy(CLIENT *clnt)
 {
-	struct cx_data *cx = (struct cx_data *)clnt->cl_p1;
-	struct rpc_dplx_rec *rec = (struct rpc_dplx_rec *)clnt->cl_p2;
-	int cu_fd = CU_DATA(cx)->cu_fd;
-	sigset_t mask, newmask;
+	struct cx_data *cx = CX_DATA(clnt);
+	struct cu_data *cu = CU_DATA(cx);
+	struct rpc_dplx_rec *rec = cx->cx_rec;
 
-	/* Handle our own signal mask here, the signal section is
-	 * larger than the wait (not 100% clear why) */
-	sigfillset(&newmask);
-	thr_sigsetmask(SIG_SETMASK, &newmask, &mask);
-
-	/* barrier both channels */
-	rpc_dplx_swc(clnt, rpc_flag_clear);
-	rpc_dplx_rwc(clnt, rpc_flag_clear);
-
-	if (CU_DATA(cx)->cu_closeit)
-		(void)close(cu_fd);
-	XDR_DESTROY(&(CU_DATA(cx)->cu_outxdrs));
-
-	/* signal both channels */
-	rpc_dplx_ssc(clnt, RPC_DPLX_FLAG_NONE);
-	rpc_dplx_rsc(clnt, RPC_DPLX_FLAG_NONE);
+	XDR_DESTROY(&cu->cu_outxdrs);
 
 	/* release */
-	rpc_dplx_unref(rec, RPC_DPLX_FLAG_NONE);
+	SVC_RELEASE(&rec->xprt, SVC_RELEASE_FLAG_NONE);
 	free_cx_data(cx);
-
-	if (clnt->cl_netid && clnt->cl_netid[0])
-		mem_free(clnt->cl_netid, strlen(clnt->cl_netid) + 1);
-	if (clnt->cl_tp && clnt->cl_tp[0])
-		mem_free(clnt->cl_tp, strlen(clnt->cl_tp) + 1);
-	mem_free(clnt, sizeof(CLIENT));
-	thr_sigsetmask(SIG_SETMASK, &mask, NULL);
 }
 
 static struct clnt_ops *
