@@ -80,15 +80,15 @@
 #include "svc_ioq.h"
 
 int generic_read_vc(XDR *, void *, void *, int);
-int generic_write_vc(XDR *, void *, void *, int);
 
-static void svc_vc_rendezvous_ops(SVCXPRT *, u_int);
-static void svc_vc_ops(SVCXPRT *, u_int);
+static void svc_vc_rendezvous_ops(SVCXPRT *);
+static void svc_vc_ops(SVCXPRT *);
 static void svc_vc_override_ops(SVCXPRT *, SVCXPRT *);
 
 bool __svc_clean_idle2(int, bool);
 
-static SVCXPRT *makefd_xprt(int, u_int, u_int, bool *);
+static SVCXPRT *makefd_xprt(const int, const u_int, const u_int,
+			    struct __rpc_sockinfo *, u_int);
 
 extern pthread_mutex_t svc_ctr_lock;
 
@@ -97,355 +97,268 @@ extern pthread_mutex_t svc_ctr_lock;
  * xprt = svc_vc_ncreate(sock, send_buf_size, recv_buf_size);
  *
  * Creates, registers, and returns a (rpc) tcp based transport.
- * Once *xprt is initialized, it is registered as a transport
- * see (svc.h, xprt_register).  This routine returns
- * a NULL if a problem occurred.
+ * If a problem occurred, this routine returns a NULL.
  *
  * Since streams do buffered io similar to stdio, the caller can specify
  * how big the send and receive buffers are via the second and third parms;
  * 0 => use the system default.
  *
- * Added svc_vc_ncreate2 with flags argument, has the behavior of the
- * original function if flags are SVC_VC_FLAG_NONE (0).
+ * Added svc_vc_ncreatef with flags argument, has the behavior of the
+ * original function with flags SVC_CREATE_FLAG_CLOSE.
  *
  */
+static void
+svc_vc_xprt_free(struct svc_vc_xprt *xd)
+{
+	rpc_dplx_rec_destroy(&xd->sx_dr);
+	mutex_destroy(&xd->sx_dr.xprt.xp_lock);
+	mutex_destroy(&xd->sx_dr.xprt.xp_auth_lock);
+
+#if defined(HAVE_BLKIN)
+	if (xd->sx_dr.xprt.blkin.svc_name)
+		mem_free(xd->sx_dr.xprt.blkin.svc_name, 2*INET6_ADDRSTRLEN);
+#endif
+	mem_free(xd, sizeof(struct svc_vc_xprt));
+}
+
+static struct svc_vc_xprt *
+svc_vc_xprt_zalloc(void)
+{
+	struct svc_vc_xprt *xd = mem_zalloc(sizeof(struct svc_vc_xprt));
+
+	/* Init SVCXPRT locks, etc */
+	mutex_init(&xd->sx_dr.xprt.xp_lock, NULL);
+	mutex_init(&xd->sx_dr.xprt.xp_auth_lock, NULL);
+/*	TAILQ_INIT_ENTRY(&xd->sx_dr.xprt, xp_evq); sets NULL */
+	rpc_dplx_rec_init(&xd->sx_dr);
+
+	xd->sx.strm_stat = XPRT_IDLE;
+	xd->sx_dr.xprt.xp_refs = 1;
+	return (xd);
+}
+
+void
+svc_vc_xprt_setup(SVCXPRT **sxpp)
+{
+	if (unlikely(*sxpp)) {
+		svc_vc_xprt_free(VC_DR(REC_XPRT(*sxpp)));
+		*sxpp = NULL;
+	} else {
+		struct svc_vc_xprt *xd = svc_vc_xprt_zalloc();
+
+		*sxpp = &xd->sx_dr.xprt;
+	}
+}
+
 SVCXPRT *
-svc_vc_ncreate2(int fd, u_int sendsize, u_int recvsize, u_int flags)
+svc_vc_ncreatef(const int fd, const u_int sendsz, const u_int recvsz,
+		const uint32_t flags)
 {
-	SVCXPRT *xprt = NULL;
-	struct cf_rendezvous *rdvs;
 	struct __rpc_sockinfo si;
-	struct sockaddr_storage sslocal;
-	struct sockaddr *salocal;
-	struct sockaddr_in *salocal_in;
-	struct sockaddr_in6 *salocal_in6;
-	struct rpc_dplx_rec *rec = NULL;
-	struct x_vc_data *xd = NULL;
+	SVCXPRT *xprt;
+	struct rpc_dplx_rec *rec;
+	struct svc_vc_xprt *xd;
 	const char *netid;
-	uint32_t oflags;
-	socklen_t slen;
+	u_int recvsize;
+	u_int sendsize;
+	u_int xp_flags;
+	int rc;
 
-	if (!__rpc_fd2sockinfo(fd, &si))
-		return NULL;
+	/* atomically find or create shared fd state; ref+1; locked */
+	xprt = svc_xprt_lookup(fd, svc_vc_xprt_setup);
+	if (!xprt) {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
+			"%s: fd %d svc_xprt_lookup failed",
+			__func__, fd);
+		return (NULL);
+	}
+	rec = REC_XPRT(xprt);
 
-	if (!__rpc_sockinfo2netid(&si, &netid))
-		return NULL;
+	xp_flags = atomic_postset_uint16_t_bits(&xprt->xp_flags,
+						(flags & SVC_XPRT_FLAG_CLOSE)
+						| SVC_XPRT_FLAG_INITIALIZED);
+	if (xp_flags & SVC_XPRT_FLAG_INITIALIZED) {
+		rpc_dplx_rui(rec);
+		XPRT_TRACE(xprt, __func__, __func__, __LINE__);
+		return (xprt);
+	}
 
+	if (!__rpc_fd2sockinfo(fd, &si)) {
+		atomic_clear_uint16_t_bits(&xprt->xp_flags,
+					   SVC_XPRT_FLAG_INITIALIZED);
+		rpc_dplx_rui(rec);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d could not get transport information",
+			__func__, fd);
+		return (NULL);
+	}
+
+	if (!__rpc_sockinfo2netid(&si, &netid)) {
+		atomic_clear_uint16_t_bits(&xprt->xp_flags,
+					   SVC_XPRT_FLAG_INITIALIZED);
+		rpc_dplx_rui(rec);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d could not get network information",
+			__func__, fd);
+		return (NULL);
+	}
+	xd = VC_DR(rec);
+
+	opr_rbtree_init(&xd->cx.calls.t, call_xid_cmpf);
+/*	xd->cx.calls.xid = 0;	next call xid is 1 */
+
+	/*
+	 * Find the receive and the send size
+	 */
+	sendsize = __rpc_get_t_size(si.si_af, si.si_proto, (int)sendsz);
+	recvsize = __rpc_get_t_size(si.si_af, si.si_proto, (int)recvsz);
+	/*
+	 * Should be multiple of 4 for XDR.
+	 */
+	xd->shared.sendsz = ((sendsize + 3) / 4) * 4;
+	xd->shared.recvsz = ((recvsize + 3) / 4) * 4;
+	xd->sx.maxrec = __svc_maxrec;
+
+	/* duplex streams are not used by the rendevous transport */
+
+	svc_vc_rendezvous_ops(xprt);
 #ifdef RPC_VSOCK
-	flags |= (si.si_af == AF_VSOCK) ? SVC_VC_CREATE_VSOCK : 0;
+	if (si.si_af == AF_VSOCK)
+		 xprt->xp_type = XPRT_VSOCK_RENDEZVOUS;
 #endif /* VSOCK */
 
-	rdvs = mem_alloc(sizeof(struct cf_rendezvous));
-	rdvs->sendsize = __rpc_get_t_size(si.si_af, si.si_proto, (int)sendsize);
-	rdvs->recvsize = __rpc_get_t_size(si.si_af, si.si_proto, (int)recvsize);
-	rdvs->maxrec = __svc_maxrec;
-
-	/* atomically find or create shared fd state */
-	rec = rpc_dplx_lookup_rec(fd, RPC_DPLX_LKP_IFLAG_LOCKREC, &oflags);
-	if (!rec) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_vc: makefd_xprt: rpc_dplx_lookup_rec failed");
-		goto err;
-	}
-
-	/* attach shared state */
-	if ((oflags & RPC_DPLX_LKP_OFLAG_ALLOC) || (!rec->hdl.xd)) {
-		xd = rec->hdl.xd = alloc_x_vc_data();
-		if (xd == NULL) {
-			__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-				"svc_vc: makefd_xprt: out of memory");
-			goto err;
-		}
-
-		xd->rec = rec;
-
-		/* XXX tracks outstanding calls */
-		opr_rbtree_init(&xd->cx.calls.t, call_xid_cmpf);
-		xd->cx.calls.xid = 0;	/* next call xid is 1 */
-		xd->refcnt = 1;
-
-		xd->shared.sendsz =
-		    __rpc_get_t_size(si.si_af, si.si_proto, (int)sendsize);
-		xd->shared.recvsz =
-		    __rpc_get_t_size(si.si_af, si.si_proto, (int)recvsize);
-
-		/* duplex streams are not used by the rendevous transport */
-		memset(&xd->shared.xdrs_in, 0, sizeof xd->shared.xdrs_in);
-		memset(&xd->shared.xdrs_out, 0, sizeof xd->shared.xdrs_out);
-	} else {
-		xd = (struct x_vc_data *)rec->hdl.xd;
-		/* dont return destroyed xprts */
-		if (!(xd->flags & X_VC_DATA_FLAG_SVC_DESTROYED)) {
-			if (rec->hdl.xprt) {
-				xprt = rec->hdl.xprt;
-				/* inc xprt refcnt */
-				SVC_REF(xprt, SVC_REF_FLAG_NONE);
-				mem_free(rdvs, sizeof(struct cf_rendezvous));
-				goto done;
-			} else
-				++(xd->refcnt);
-		}
-		/* return extra ref */
-		rpc_dplx_unref(rec,
-			       RPC_DPLX_FLAG_LOCKED | RPC_DPLX_FLAG_UNLOCK);
-	}
-
-	xprt = mem_zalloc(sizeof(SVCXPRT));
-	xprt->xp_flags = SVC_XPRT_FLAG_NONE;
-	xprt->xp_refs = 1;
-	svc_vc_rendezvous_ops(xprt, flags);
-	xprt->xp_p1 = rdvs;
-	xprt->xp_p2 = xd;
-	xprt->xp_p5 = rec;
-	xprt->xp_fd = fd;
-	mutex_init(&xprt->xp_lock, NULL);
-
 	/* caller should know what it's doing */
-	if (flags & SVC_VC_CREATE_LISTEN)
-		listen(fd, SOMAXCONN);
-
-	slen = sizeof(struct sockaddr_storage);
-	if (getsockname(fd, (struct sockaddr *)(void *)&sslocal, &slen) < 0) {
+	if (flags & SVC_CREATE_FLAG_LISTEN) {
 		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_vc_create: could not retrieve local addr");
-		goto err;
+			"%s: fd %d listen",
+			 __func__, fd);
+		listen(fd, SOMAXCONN);
 	}
 
-	/* XXX following breaks strict aliasing? */
-	salocal = (struct sockaddr *)&sslocal;
-	switch (salocal->sa_family) {
-	case AF_INET:
-		salocal_in = (struct sockaddr_in *)salocal;
-		xprt->xp_port = ntohs(salocal_in->sin_port);
-		break;
-	case AF_INET6:
-		salocal_in6 = (struct sockaddr_in6 *)salocal;
-		xprt->xp_port = ntohs(salocal_in6->sin6_port);
-		break;
+	__rpc_address_setup(&xprt->xp_local);
+	rc = getsockname(fd, xprt->xp_local.nb.buf, &xprt->xp_local.nb.len);
+	if (rc < 0) {
+		atomic_clear_uint16_t_bits(&xprt->xp_flags,
+					   SVC_XPRT_FLAG_INITIALIZED);
+		rpc_dplx_rui(rec);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d getsockname failed (%d)",
+			 __func__, fd, rc);
+		return (NULL);
 	}
-	__rpc_set_address(&xprt->xp_local, &sslocal, slen);
 
 	xprt->xp_netid = mem_strdup(netid);
 
-	/* make reachable from rec */
-	rec->hdl.xprt = xprt;
-
 	/* release rec */
-	REC_UNLOCK(rec);
+	rpc_dplx_rui(rec);
+	XPRT_TRACE(xprt, __func__, __func__, __LINE__);
 
-	/* make reachable from xprt list */
-	svc_rqst_init_xprt(xprt);
-
-	/* conditional xprt_register */
-	if ((!(__svc_params->flags & SVC_FLAG_NOREG_XPRTS))
-	    && (!(flags & SVC_VC_CREATE_XPRT_NOREG)))
-		xprt_register(xprt);
+	/* Conditional register */
+	if ((!(__svc_params->flags & SVC_FLAG_NOREG_XPRTS)
+	     && !(flags & SVC_CREATE_FLAG_XPRT_NOREG))
+	    || (flags & SVC_CREATE_FLAG_XPRT_DOREG))
+		svc_rqst_evchan_reg(__svc_params->ev_u.evchan.id, xprt,
+				    SVC_RQST_FLAG_CHAN_AFFINITY);
 
 #if defined(HAVE_BLKIN)
 	__rpc_set_blkin_endpoint(xprt, "svc_vc");
 #endif
 
- done:
 	return (xprt);
-
- err:
-	mem_free(rdvs, sizeof(struct cf_rendezvous));
-
-	if (xprt) {
-#if defined(HAVE_BLKIN)
-		if (xprt->blkin.svc_name)
-			mem_free(xprt->blkin.svc_name, 2*INET6_ADDRSTRLEN);
-#endif
-		mem_free(xprt, sizeof(SVCXPRT));
-	}
-
-	if (rec) {
-		rpc_dplx_unref(rec,
-			       RPC_DPLX_FLAG_LOCKED | RPC_DPLX_FLAG_UNLOCK);
-	}
-
-	return (NULL);
-}
-
-SVCXPRT *
-svc_vc_ncreate(int fd, u_int sendsize, u_int recvsize)
-{
-	return (svc_vc_ncreate2(fd, sendsize, recvsize, SVC_VC_CREATE_NONE));
-}
-
-/*
- * Like svtcp_ncreate(), except the routine takes any *open* UNIX file
- * descriptor as its first input.
- */
-SVCXPRT *
-svc_fd_ncreate(int fd, u_int sendsize, u_int recvsize)
-{
-	struct sockaddr_storage ss;
-	socklen_t slen;
-	SVCXPRT *xprt;
-	bool xprt_allocd;
-
-	assert(fd != -1);
-
-	xprt = makefd_xprt(fd, sendsize, recvsize, &xprt_allocd);
-	if ((!xprt) || (!xprt_allocd))	/* ref'd existing xprt handle */
-		goto done;
-
-	/* conditional xprt_register */
-	if (!(__svc_params->flags & SVC_FLAG_NOREG_XPRTS))
-		xprt_register(xprt);
-
-	slen = sizeof(struct sockaddr_storage);
-	if (getsockname(fd, (struct sockaddr *)(void *)&ss, &slen) < 0) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_fd_create: could not retrieve local addr");
-		goto freedata;
-	}
-	__rpc_set_address(&xprt->xp_local, &ss, slen);
-
-	slen = sizeof(struct sockaddr_storage);
-	if (getpeername(fd, (struct sockaddr *)(void *)&ss, &slen) < 0) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_fd_create: could not retrieve remote addr");
-		goto freedata;
-	}
-	__rpc_set_address(&xprt->xp_remote, &ss, slen);
-
- done:
-	return (xprt);
-
- freedata:
-	return (NULL);
 }
 
 /*
  * Like sv_fd_ncreate(), except export flags for additional control.
  */
 SVCXPRT *
-svc_fd_ncreate2(int fd, u_int sendsize, u_int recvsize, u_int flags)
+svc_fd_ncreatef(const int fd, const u_int sendsize, const u_int recvsize,
+		const uint32_t flags)
 {
-	struct sockaddr_storage ss;
-	struct sockaddr *sa = (struct sockaddr *)(void *)&ss;
-	socklen_t slen;
 	SVCXPRT *xprt;
-	bool xprt_allocd;
+	struct __rpc_sockinfo si;
+	int rc;
 
 	assert(fd != -1);
 
-	xprt = makefd_xprt(fd, sendsize, recvsize, &xprt_allocd);
-	if ((!xprt) || (!xprt_allocd))	/* ref'd existing xprt handle */
+	xprt = makefd_xprt(fd, sendsize, recvsize, &si,
+			   flags & SVC_XPRT_FLAG_CLOSE);
+	if ((!xprt) || (!(xprt->xp_flags & SVC_XPRT_FLAG_INITIAL)))
 		return (xprt);
 
-	slen = sizeof(struct sockaddr_storage);
-	if (getsockname(fd, sa, &slen) < 0) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_fd_ncreate: could not retrieve local addr");
+	__rpc_address_setup(&xprt->xp_local);
+	rc = getsockname(fd, xprt->xp_local.nb.buf, &xprt->xp_local.nb.len);
+	if (rc < 0) {
+		xprt->xp_local.nb.len = sizeof(struct sockaddr_storage);
+		memset(xprt->xp_local.nb.buf, 0xfe, xprt->xp_local.nb.len);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d getsockname failed (%d)",
+			 __func__, fd, rc);
 		return (NULL);
 	}
-	__rpc_set_address(&xprt->xp_local, &ss, slen);
 
-	slen = sizeof(struct sockaddr_storage);
-	if (getpeername(fd, (struct sockaddr *)(void *)&ss, &slen) < 0) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_fd_ncreate: could not retrieve remote addr");
+	__rpc_address_setup(&xprt->xp_remote);
+	rc = getpeername(fd, xprt->xp_remote.nb.buf, &xprt->xp_remote.nb.len);
+	if (rc < 0) {
+		xprt->xp_remote.nb.len = sizeof(struct sockaddr_storage);
+		memset(xprt->xp_remote.nb.buf, 0xfe, xprt->xp_remote.nb.len);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d getpeername failed (%d)",
+			 __func__, fd, rc);
 		return (NULL);
 	}
-	__rpc_set_address(&xprt->xp_remote, &ss, slen);
+	XPRT_TRACE(xprt, __func__, __func__, __LINE__);
 
-	/* conditional xprt_register */
-	if ((!(__svc_params->flags & SVC_FLAG_NOREG_XPRTS))
-	    && (!(flags & SVC_VC_CREATE_XPRT_NOREG)))
-		xprt_register(xprt);
+	/* Conditional register */
+	if ((!(__svc_params->flags & SVC_FLAG_NOREG_XPRTS)
+	     && !(flags & SVC_CREATE_FLAG_XPRT_NOREG))
+	    || (flags & SVC_CREATE_FLAG_XPRT_DOREG))
+		svc_rqst_evchan_reg(__svc_params->ev_u.evchan.id, xprt,
+				    SVC_RQST_FLAG_CHAN_AFFINITY);
+
+#if defined(HAVE_BLKIN)
+	__rpc_set_blkin_endpoint(xprt, "svc_vc");
+#endif
 
 	return (xprt);
 }
 
 static SVCXPRT *
-	makefd_xprt(int fd, u_int sendsz, u_int recvsz, bool *allocated)
+makefd_xprt(const int fd, const u_int sendsz, const u_int recvsz,
+	    struct __rpc_sockinfo *si, u_int flags)
 {
-	SVCXPRT *xprt = NULL;
-	struct x_vc_data *xd = NULL;
+	SVCXPRT *xprt;
+	struct svc_vc_xprt *xd;
 	struct rpc_dplx_rec *rec;
-	struct __rpc_sockinfo si;
 	const char *netid;
-	uint32_t oflags;
-	bool newxd = false;
-	*allocated = false;
+	u_int recvsize;
+	u_int sendsize;
+	u_int xp_flags;
 
 	assert(fd != -1);
 
 	if (!svc_vc_new_conn_ok()) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"%s: makefd_xprt: max_connections exceeded\n",
-			__func__);
-		goto done;
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d max_connections exceeded\n",
+			__func__, fd);
+		return (NULL);
 	}
 
-	/* atomically find or create shared fd state */
-	rec = rpc_dplx_lookup_rec(fd, RPC_DPLX_LKP_IFLAG_LOCKREC, &oflags);
-	if (!rec) {
+	/* atomically find or create shared fd state; ref+1; locked */
+	xprt = svc_xprt_lookup(fd, svc_vc_xprt_setup);
+	if (!xprt) {
 		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"svc_vc: makefd_xprt: rpc_dplx_lookup_rec failed");
-		goto done;
+			"%s: fd %d svc_xprt_lookup failed",
+			__func__, fd);
+		return (NULL);
 	}
+	rec = REC_XPRT(xprt);
 
-	/* attach shared state */
-	if ((oflags & RPC_DPLX_LKP_OFLAG_ALLOC) || (!rec->hdl.xd)) {
-		newxd = true;
-		xd = rec->hdl.xd = alloc_x_vc_data();
-		if (xd == NULL) {
-			__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-				"svc_vc: makefd_xprt: out of memory");
-			/* return extra ref */
-			rpc_dplx_unref(rec,
-				       RPC_DPLX_FLAG_LOCKED |
-				       RPC_DPLX_FLAG_UNLOCK);
-			mem_free(xprt, sizeof(SVCXPRT));
-			goto done;
-		}
-
-		xd->rec = rec;
-
-		/* XXX tracks outstanding calls */
-		opr_rbtree_init(&xd->cx.calls.t, call_xid_cmpf);
-		xd->cx.calls.xid = 0;	/* next call xid is 1 */
-		xd->refcnt = 1;
-
-		if (__rpc_fd2sockinfo(fd, &si)) {
-			xd->shared.sendsz =
-				__rpc_get_t_size(
-					si.si_af, si.si_proto, (int)sendsz);
-			xd->shared.recvsz =
-				__rpc_get_t_size(
-					si.si_af, si.si_proto, (int)recvsz);
-		}
-
-		/* duplex streams */
-		xdr_inrec_create(&(xd->shared.xdrs_in), recvsz, xd,
-				 generic_read_vc);
-		xd->shared.xdrs_in.x_op = XDR_DECODE;
-
-		xdrrec_create(&(xd->shared.xdrs_out), sendsz, recvsz, xd,
-			      generic_read_vc, generic_write_vc);
-		xd->shared.xdrs_out.x_op = XDR_ENCODE;
-	} else {
-		xd = (struct x_vc_data *)rec->hdl.xd;
-		/* dont return destroyed xprts */
-		if (!(xd->flags & X_VC_DATA_FLAG_SVC_DESTROYED)) {
-			if (rec->hdl.xprt) {
-				xprt = rec->hdl.xprt;
-				/* inc xprt refcnt */
-				SVC_REF(xprt, SVC_REF_FLAG_NONE);
-			} else
-				++(xd->refcnt);
-		}
-		/* return extra ref */
-		rpc_dplx_unref(rec,
-			       RPC_DPLX_FLAG_LOCKED | RPC_DPLX_FLAG_UNLOCK);
-		*allocated = FALSE;
-
-		/* return ref'd xprt */
-		goto done_xprt;
+	xp_flags = atomic_postset_uint16_t_bits(&xprt->xp_flags, flags
+						| SVC_XPRT_FLAG_INITIALIZED);
+	if (xp_flags & SVC_XPRT_FLAG_INITIALIZED) {
+		rpc_dplx_rui(rec);
+		XPRT_TRACE(xprt, __func__, __func__, __LINE__);
+		return (xprt);
 	}
 
 	/* XXX bi-directional?  initially I had assumed that explicit
@@ -453,64 +366,80 @@ static SVCXPRT *
 	 * handle of the other type, but perhaps it is more natural to
 	 * just discover it
 	 */
+	if (!__rpc_fd2sockinfo(fd, si)) {
+		atomic_clear_uint16_t_bits(&xprt->xp_flags,
+					   SVC_XPRT_FLAG_INITIALIZED);
+		rpc_dplx_rui(rec);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d could not get transport information",
+			__func__, fd);
+		return (NULL);
+	}
 
-	/* new xprt (the common case) */
-	xprt = mem_zalloc(sizeof(SVCXPRT));
+	if (!__rpc_sockinfo2netid(si, &netid)) {
+		atomic_clear_uint16_t_bits(&xprt->xp_flags,
+					   SVC_XPRT_FLAG_INITIALIZED);
+		rpc_dplx_rui(rec);
+		__warnx(TIRPC_DEBUG_FLAG_ERROR,
+			"%s: fd %d could not get network information",
+			__func__, fd);
+		return (NULL);
+	}
+	xd = VC_DR(rec);
 
-	*allocated = TRUE;
-	xprt->xp_p5 = rec;
-	mutex_init(&xprt->xp_lock, NULL);
-	/* XXX take xp_lock? */
-	mutex_init(&xprt->xp_auth_lock, NULL);
-	xprt->xp_refs = 1;
-	xprt->xp_fd = fd;
+	opr_rbtree_init(&xd->cx.calls.t, call_xid_cmpf);
+/*	xd->cx.calls.xid = 0;	next call xid is 1 */
+
+	/*
+	 * Find the receive and the send size
+	 */
+	sendsize = __rpc_get_t_size(si->si_af, si->si_proto, (int)sendsz);
+	recvsize = __rpc_get_t_size(si->si_af, si->si_proto, (int)recvsz);
+	/*
+	 * Should be multiple of 4 for XDR.
+	 */
+	xd->shared.sendsz = ((sendsize + 3) / 4) * 4;
+	xd->shared.recvsz = ((recvsize + 3) / 4) * 4;
+	xd->sx.maxrec = __svc_maxrec;
+
+	/* duplex streams */
+	xdr_inrec_create(&(xd->shared.xdrs_in), xd->shared.recvsz, xd,
+			 generic_read_vc);
+	xd->shared.xdrs_in.x_op = XDR_DECODE;
 
 	/* the SVCXPRT created in svc_vc_create accepts new connections
 	 * in its xp_recv op, the rendezvous_request method, but xprt is
 	 * a call channel */
-	u_int ops_flags =
+	svc_vc_ops(xprt);
 #ifdef RPC_VSOCK
-		(si.si_af == AF_VSOCK) ? SVC_VC_CREATE_VSOCK :
+	if (si->si_af == AF_VSOCK)
+		 xprt->xp_type = XPRT_VSOCK;
 #endif /* VSOCK */
-		0;
-	svc_vc_ops(xprt, ops_flags);
 
-	xd->sx.strm_stat = XPRT_IDLE;
-
-	xprt->xp_p1 = xd;
-	if (newxd /* ensures valid si */ && __rpc_sockinfo2netid(&si, &netid))
-		xprt->xp_netid = mem_strdup(netid);
-
-	/* make reachable from rec */
-	rec->hdl.xprt = xprt;
+	xprt->xp_netid = mem_strdup(netid);
 
 	/* release */
-	REC_UNLOCK(rec);
+	rpc_dplx_rui(rec);
+	XPRT_TRACE(xprt, __func__, __func__, __LINE__);
 
-	/* Make reachable from xprt list.  Registration deferred. */
-	svc_rqst_init_xprt(xprt);
-
-done_xprt:
-done:
 	return (xprt);
 }
 
  /*ARGSUSED*/
 static bool
-rendezvous_request(SVCXPRT *xprt, struct svc_req *req)
+rendezvous_request(struct svc_req *req)
 {
-	int fd;
-	socklen_t len;
-	struct cf_rendezvous *rdvs;
-	struct x_vc_data *xd;
+	SVCXPRT *xprt = req->rq_xprt;
+	struct svc_vc_xprt *req_xd = VC_DR(REC_XPRT(xprt));
+	SVCXPRT *newxprt;
+	struct svc_vc_xprt *xd;
 	struct sockaddr_storage addr;
 	struct __rpc_sockinfo si;
-	socklen_t slen;
-	SVCXPRT *newxprt;
-	bool xprt_allocd;
+	int fd;
+	int rc;
+	socklen_t len;
 	static int n = 1;
 
-	rdvs = (struct cf_rendezvous *)xprt->xp_p1;
  again:
 	len = sizeof(addr);
 	fd = accept(xprt->xp_fd, (struct sockaddr *)(void *)&addr, &len);
@@ -541,8 +470,9 @@ rendezvous_request(SVCXPRT *xprt, struct svc_req *req)
 	/*
 	 * make a new transport (re-uses xprt)
 	 */
-	newxprt = makefd_xprt(fd, rdvs->sendsize, rdvs->recvsize, &xprt_allocd);
-	if ((!newxprt) || (!xprt_allocd)) /* ref'd existing xprt handle */
+	newxprt = makefd_xprt(fd, req_xd->shared.sendsz, req_xd->shared.recvsz,
+			      &si, SVC_XPRT_FLAG_CLOSE);
+	if ((!newxprt) || (!(newxprt->xp_flags & SVC_XPRT_FLAG_INITIAL)))
 		return (FALSE);
 
 	/*
@@ -553,32 +483,38 @@ rendezvous_request(SVCXPRT *xprt, struct svc_req *req)
 	/* move xprt_register() out of makefd_xprt */
 	(void)svc_rqst_xprt_register(xprt, newxprt);
 
-	__rpc_set_address(&newxprt->xp_remote, &addr, len);
+	__rpc_address_setup(&newxprt->xp_remote);
+	memcpy(newxprt->xp_remote.nb.buf, &addr, len);
+	newxprt->xp_remote.nb.len = len;
 	XPRT_TRACE(newxprt, __func__, __func__, __LINE__);
 
 	/* XXX fvdl - is this useful? (Yes.  Matt) */
-	if (__rpc_fd2sockinfo(fd, &si) && si.si_proto == IPPROTO_TCP) {
+	if (si.si_proto == IPPROTO_TCP) {
 		len = 1;
 		(void) setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &len,
 				  sizeof(len));
 	}
 
-	slen = sizeof(struct sockaddr_storage);
-	if (getsockname(fd, (struct sockaddr *)(void *)&addr, &slen) < 0) {
+	__rpc_address_setup(&newxprt->xp_local);
+	rc = getsockname(fd, newxprt->xp_local.nb.buf,
+			 &newxprt->xp_local.nb.len);
+	if (rc < 0) {
+		newxprt->xp_local.nb.len = sizeof(struct sockaddr_storage);
+		memset(newxprt->xp_local.nb.buf, 0xfe,
+		       newxprt->xp_local.nb.len);
 		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"%s: could not retrieve local addr", __func__);
-	} else {
-		__rpc_set_address(&newxprt->xp_local, &addr, slen);
+			"%s: fd %d getsockname failed (%d)",
+			 __func__, fd, rc);
 	}
 
 #if defined(HAVE_BLKIN)
 	__rpc_set_blkin_endpoint(newxprt, "svc_vc");
 #endif
 
-	xd = (struct x_vc_data *)newxprt->xp_p1;
-	xd->shared.recvsz = rdvs->recvsize;
-	xd->shared.sendsz = rdvs->sendsize;
-	xd->sx.maxrec = rdvs->maxrec;
+	xd = VC_DR(REC_XPRT(newxprt));
+	xd->shared.recvsz = req_xd->shared.recvsz;
+	xd->shared.sendsz = req_xd->shared.sendsz;
+	xd->sx.maxrec = req_xd->sx.maxrec;
 
 #if 0  /* XXX vrec wont support atm (and it seems to need work) */
 	if (cd->maxrec != 0) {
@@ -615,29 +551,23 @@ rendezvous_stat(SVCXPRT *xprt)
 }
 
 /* XXX pending further unification
- *
- * note:  currently, rdvs xprt handles have a rec structure,
- * but no xd structure, etc.
- * (they do too have an xd -- to track "destroyed" flag -- fixme?)
- *
  */
 
 static void
-svc_rdvs_destroy(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
+svc_vc_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 {
-	struct cf_rendezvous *rdvs = (struct cf_rendezvous *)xprt->xp_p1;
-	struct x_vc_data *xd = (struct x_vc_data *)xprt->xp_p2;
-	struct rpc_dplx_rec *rec = (struct rpc_dplx_rec *)xprt->xp_p5;
+	struct svc_vc_xprt *xd = VC_DR(REC_XPRT(xprt));
 
 	/* clears xprt from the xprt table (eg, idle scans) */
-	xprt_unregister(xprt);
+	svc_rqst_xprt_unregister(xprt);
 
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
 		"%s() %p xp_refs %" PRIu32
 		" should actually destroy things @ %s:%d",
 		__func__, xprt, xprt->xp_refs, tag, line);
 
-	if (xprt->xp_fd != RPC_ANYFD)
+	if ((xprt->xp_flags & SVC_XPRT_FLAG_CLOSE)
+	    && xprt->xp_fd != RPC_ANYFD)
 		(void)close(xprt->xp_fd);
 
 	if (xprt->xp_ops->xp_free_user_data) {
@@ -645,66 +575,24 @@ svc_rdvs_destroy(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 		xprt->xp_ops->xp_free_user_data(xprt);
 	}
 
-	REC_LOCK(rec);
-
-	mutex_destroy(&xprt->xp_lock);
-
 	if (xprt->xp_tp)
 		mem_free(xprt->xp_tp, 0);
 	if (xprt->xp_netid)
 		mem_free(xprt->xp_netid, 0);
 
-	mem_free(rdvs, sizeof(struct cf_rendezvous));
-	mem_free(xprt, sizeof(SVCXPRT));
-
-	(void)rpc_dplx_unref(rec, RPC_DPLX_FLAG_LOCKED | RPC_DPLX_FLAG_UNLOCK);
-
-	if (xd->refcnt == 0) {
-		XDR_DESTROY(&xd->shared.xdrs_in);
-		XDR_DESTROY(&xd->shared.xdrs_out);
-		free_x_vc_data(xd);
-	}
+	svc_vc_xprt_free(xd);
 }
 
 static void
 svc_vc_destroy(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 {
-	struct x_vc_data *xd = (struct x_vc_data *)xprt->xp_p1;
-	struct rpc_dplx_rec *rec = xd->rec;
-	uint32_t xd_refcnt;
-
 	/* connection tracking--decrement now, he's dead jim */
 	svc_vc_dec_nconns();
 
-	/* clears xprt from the xprt table (eg, idle scans) */
-	xprt_unregister(xprt);
+	/* destroy shared XDR record streams (once) */
+	XDR_DESTROY(&(VC_DR(REC_XPRT(xprt))->shared.xdrs_in));
 
-	/* bidirectional */
-	REC_LOCK(rec);
-	xd->flags |= X_VC_DATA_FLAG_SVC_DESTROYED;
-	xd_refcnt = --(xd->refcnt);
-
-	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-		"%s: postfinalize %p xp_refs %" PRIu32
-		" xd_refcnt %u",
-		__func__, xprt, xprt->xp_refs, xd_refcnt);
-
-	/* conditional destroy */
-	if (xd_refcnt == 0) {
-		__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-			"%s: %p xp_refs %" PRIu32
-			" xd_refcnt %" PRIu32
-			" calling vc_shared_destroy @ %s:%d",
-			__func__, xprt, xprt->xp_refs, xd_refcnt, tag, line);
-		vc_shared_destroy(xd);	/* RECLOCKED */
-	} else {
-		__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-			"%s: %p xp_refs %" PRIu32
-			" xd_refcnt %" PRIu32
-			" omit vc_shared_destroy @ %s:%d",
-			__func__, xprt, xprt->xp_refs, xd_refcnt, tag, line);
-		REC_UNLOCK(rec);
-	}
+	svc_vc_destroy_it(xprt, flags, tag, line);
 }
 
 extern mutex_t ops_lock;
@@ -779,17 +667,14 @@ svc_vc_control(SVCXPRT *xprt, const u_int rq, void *in)
 static bool
 svc_vc_rendezvous_control(SVCXPRT *xprt, const u_int rq, void *in)
 {
-	struct cf_rendezvous *cfp;
+	struct svc_vc_xprt *xd = VC_DR(REC_XPRT(xprt));
 
-	cfp = (struct cf_rendezvous *)xprt->xp_p1;
-	if (cfp == NULL)
-		return (FALSE);
 	switch (rq) {
 	case SVCGET_CONNMAXREC:
-		*(int *)in = cfp->maxrec;
+		*(int *)in = xd->sx.maxrec;
 		break;
 	case SVCSET_CONNMAXREC:
-		cfp->maxrec = *(int *)in;
+		xd->sx.maxrec = *(int *)in;
 		break;
 	case SVCGET_XP_RECV:
 		mutex_lock(&ops_lock);
@@ -850,29 +735,46 @@ svc_vc_rendezvous_control(SVCXPRT *xprt, const u_int rq, void *in)
 static enum xprt_stat
 svc_vc_stat(SVCXPRT *xprt)
 {
-	struct x_vc_data *xd = (struct x_vc_data *)xprt->xp_p1;
+	struct rpc_dplx_rec *rec = REC_XPRT(xprt);
+	struct svc_vc_xprt *xd = VC_DR(rec);
+	enum xprt_stat result = XPRT_IDLE;
+	uint16_t xp_flags = atomic_postclear_uint16_t_bits(&xprt->xp_flags,
+							SVC_XPRT_FLAG_BLOCKED);
 
+	if (xp_flags & SVC_XPRT_FLAG_BLOCKED) {
+		if (xd->sx.strm_stat == XPRT_DIED)
+			result = XPRT_DIED;
+		else if (!xdr_inrec_eof(&(xd->shared.xdrs_in)))
+			result = XPRT_MOREREQS;
+		rpc_dplx_rui(rec);
+		rpc_dplx_rsi(rec);
+	}
 	if (xprt->xp_flags & SVC_XPRT_FLAG_DESTROYED)
 		return (XPRT_DESTROYED);
 
-	if (!xd)
-		return (XPRT_IDLE);
-
-	/* we hold the recv lock */
-	if (xd->sx.strm_stat == XPRT_DIED)
-		return (XPRT_DIED);
-
-	if (!xdr_inrec_eof(&(xd->shared.xdrs_in)))
-		return (XPRT_MOREREQS);
-
-	return (XPRT_IDLE);
+	return (result);
 }
 
 static bool
-svc_vc_recv(SVCXPRT *xprt, struct svc_req *req)
+svc_vc_recv(struct svc_req *req)
 {
-	struct x_vc_data *xd = (struct x_vc_data *)xprt->xp_p1;
+	SVCXPRT *xprt = req->rq_xprt;
+	struct rpc_dplx_rec *rec = REC_XPRT(xprt);
+	struct svc_vc_xprt *xd = VC_DR(rec);
 	XDR *xdrs = &(xd->shared.xdrs_in);	/* recv queue */
+	uint16_t xp_flags;
+
+	/* SVC_RECV() locks dplx_rec, that is also unlocked and locked again
+	 * for rpc_ctx.  Need to ensure we're the only one with the lock now.
+	 */
+	rpc_dplx_rli(rec);
+	do {
+		xp_flags = atomic_postset_uint16_t_bits(&xprt->xp_flags,
+							SVC_XPRT_FLAG_BLOCKED);
+		if (!(xp_flags & SVC_XPRT_FLAG_BLOCKED))
+			break;
+		rpc_dplx_rwi(rec);
+	} while (TRUE);
 
 	/* XXX assert(! cd->nonblock) */
 	if (xd->shared.nonblock) {
@@ -881,35 +783,26 @@ svc_vc_recv(SVCXPRT *xprt, struct svc_req *req)
 	}
 
 	xdrs->x_op = XDR_DECODE;
-
-	xdrs->x_lib[0] = (void *)RPC_DPLX_SVC;
 	xdrs->x_lib[1] = (void *)xprt;	/* transiently thread xprt */
 
 	/* Consumes any remaining -fragment- bytes, and clears last_frag */
 	(void)xdr_inrec_skiprecord(xdrs);
 
-	req->rq_msg = alloc_rpc_msg();
-	req->rq_clntcred = req->rq_msg->rq_cred_body;
+	rpc_msg_init(&req->rq_msg);
 
 	/* Advances to next record, will read up to 1024 bytes
 	 * into the stream. */
 	(void)xdr_inrec_readahead(xdrs, 1024);
 
-	if (xdr_dplx_decode(xdrs, req->rq_msg)) {
-		switch (req->rq_msg->rm_direction) {
+	if (xdr_dplx_decode(xdrs, &req->rq_msg)) {
+		switch (req->rq_msg.rm_direction) {
 		case CALL:
 			/* an ordinary call header */
-			req->rq_xprt = xprt;
-			req->rq_prog = req->rq_msg->rm_call.cb_prog;
-			req->rq_vers = req->rq_msg->rm_call.cb_vers;
-			req->rq_proc = req->rq_msg->rm_call.cb_proc;
-			req->rq_xid = req->rq_msg->rm_xid;
 			return (TRUE);
 			break;
 		case REPLY:
 			/* reply header (xprt OK) */
-			rpc_ctx_xfer_replymsg(xd, req->rq_msg);
-			req->rq_msg = NULL;
+			rpc_ctx_xfer_replymsg(xd, &req->rq_msg);
 			break;
 		default:
 			/* not good (but xprt OK) */
@@ -919,22 +812,22 @@ svc_vc_recv(SVCXPRT *xprt, struct svc_req *req)
 		return (FALSE);
 	}
 	__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-		"%s: xdr_dplx_msg_decode failed (will set dead)", __func__);
+		"%s: fd %d failed (will set xprt %p dead)",
+		__func__, xprt->xp_fd, xprt);
 	return (FALSE);
 }
 
 static bool
-svc_vc_freeargs(SVCXPRT *xprt, struct svc_req *req, xdrproc_t xdr_args,
-		void *args_ptr)
+svc_vc_freeargs(struct svc_req *req, xdrproc_t xdr_args, void *args_ptr)
 {
 	return xdr_free(xdr_args, args_ptr);
 }
 
 static bool
-svc_vc_getargs(SVCXPRT *xprt, struct svc_req *req,
-	       xdrproc_t xdr_args, void *args_ptr, void *u_data)
+svc_vc_getargs(struct svc_req *req, xdrproc_t xdr_args, void *args_ptr,
+	       void *u_data)
 {
-	struct x_vc_data *xd = (struct x_vc_data *)xprt->xp_p1;
+	struct svc_vc_xprt *xd = VC_DR(REC_XPRT(req->rq_xprt));
 	XDR *xdrs = &xd->shared.xdrs_in;	/* recv queue */
 	bool rslt;
 
@@ -948,15 +841,14 @@ svc_vc_getargs(SVCXPRT *xprt, struct svc_req *req,
 	if (rslt)
 		req->rq_cksum = xdr_inrec_cksum(xdrs);
 	else
-		svc_vc_freeargs(xprt, req, xdr_args, args_ptr);
+		svc_vc_freeargs(req, xdr_args, args_ptr);
 
 	return (rslt);
 }
 
 static bool
-svc_vc_reply(SVCXPRT *xprt, struct svc_req *req, struct rpc_msg *msg)
+svc_vc_reply(struct svc_req *req)
 {
-	struct x_vc_data *xd = (struct x_vc_data *)xprt->xp_p1;
 	XDR *xdrs_2;
 	xdrproc_t xdr_results;
 	caddr_t xdr_location;
@@ -964,14 +856,14 @@ svc_vc_reply(SVCXPRT *xprt, struct svc_req *req, struct rpc_msg *msg)
 	bool has_args;
 	bool gss;
 
-	if (msg->rm_reply.rp_stat == MSG_ACCEPTED
-	    && msg->rm_reply.rp_acpt.ar_stat == SUCCESS) {
+	if (req->rq_msg.rm_reply.rp_stat == MSG_ACCEPTED
+	    && req->rq_msg.rm_reply.rp_acpt.ar_stat == SUCCESS) {
 		has_args = TRUE;
-		xdr_results = msg->acpted_rply.ar_results.proc;
-		xdr_location = msg->acpted_rply.ar_results.where;
+		xdr_results = req->rq_msg.RPCM_ack.ar_results.proc;
+		xdr_location = req->rq_msg.RPCM_ack.ar_results.where;
 
-		msg->acpted_rply.ar_results.proc = (xdrproc_t) xdr_void;
-		msg->acpted_rply.ar_results.where = NULL;
+		req->rq_msg.RPCM_ack.ar_results.proc = (xdrproc_t) xdr_void;
+		req->rq_msg.RPCM_ack.ar_results.where = NULL;
 	} else {
 		has_args = FALSE;
 		xdr_results = NULL;
@@ -985,45 +877,27 @@ svc_vc_reply(SVCXPRT *xprt, struct svc_req *req, struct rpc_msg *msg)
 	 * Nb, we should probably use getpagesize() on Unix.  Need
 	 * an equivalent for Windows.
 	 */
-	gss = (req->rq_cred.oa_flavor == RPCSEC_GSS);
+	gss = (req->rq_msg.cb_cred.oa_flavor == RPCSEC_GSS);
 	xdrs_2 = xdr_ioq_create(8192 /* default segment size */ ,
 				__svc_params->svc_ioq_maxbuf + 8192,
 				gss
 				? UIO_FLAG_REALLOC | UIO_FLAG_FREE
 				: UIO_FLAG_FREE);
-	if (xdr_replymsg(xdrs_2, msg)
+	if (xdr_replymsg(xdrs_2, &req->rq_msg)
 	    && (!has_args
 		|| (req->rq_auth
 		    && SVCAUTH_WRAP(req->rq_auth, req, xdrs_2, xdr_results,
 				    xdr_location)))) {
 		rstat = TRUE;
 	}
-	svc_ioq_append(xprt, xd, xdrs_2);
+
+	xdrs_2->x_lib[1] = (void *)req->rq_xprt;
+	svc_ioq_write_now(req->rq_xprt, XIOQ(xdrs_2));
 	return (rstat);
 }
 
 static void
-svc_vc_lock(SVCXPRT *xprt, uint32_t flags, const char *func, int line)
-{
-	if (flags & XP_LOCK_RECV)
-		rpc_dplx_rlxi(xprt, func, line);
-
-	if (flags & XP_LOCK_SEND)
-		rpc_dplx_slxi(xprt, func, line);
-}
-
-static void
-svc_vc_unlock(SVCXPRT *xprt, uint32_t flags, const char *func, int line)
-{
-	if (flags & XP_LOCK_RECV)
-		rpc_dplx_rux(xprt);
-
-	if (flags & XP_LOCK_SEND)
-		rpc_dplx_sux(xprt);
-}
-
-static void
-svc_vc_ops(SVCXPRT *xprt, u_int flags)
+svc_vc_ops(SVCXPRT *xprt)
 {
 	static struct xp_ops ops;
 
@@ -1031,11 +905,6 @@ svc_vc_ops(SVCXPRT *xprt, u_int flags)
 	mutex_lock(&ops_lock);
 
 	xprt->xp_type = XPRT_TCP;
-	xprt->xp_type =
-#ifdef RPC_VSOCK
-		(flags & SVC_VC_CREATE_VSOCK) ? XPRT_VSOCK :
-#endif /* VSOCK */
-		XPRT_TCP;
 
 	if (ops.xp_recv == NULL) {
 		ops.xp_recv = svc_vc_recv;
@@ -1045,8 +914,6 @@ svc_vc_ops(SVCXPRT *xprt, u_int flags)
 		ops.xp_freeargs = svc_vc_freeargs;
 		ops.xp_destroy = svc_vc_destroy;
 		ops.xp_control = svc_vc_control;
-		ops.xp_lock = svc_vc_lock;
-		ops.xp_unlock = svc_vc_unlock;
 		ops.xp_getreq = svc_getreq_default;
 		ops.xp_dispatch = svc_dispatch_default;
 		ops.xp_recv_user_data = NULL;	/* no default */
@@ -1076,36 +943,29 @@ svc_vc_override_ops(SVCXPRT *xprt, SVCXPRT *newxprt)
 }
 
 static void
-svc_vc_rendezvous_ops(SVCXPRT *xprt, u_int flags)
+svc_vc_rendezvous_ops(SVCXPRT *xprt)
 {
 	static struct xp_ops ops;
 	extern mutex_t ops_lock;
 
 	mutex_lock(&ops_lock);
 
-	xprt->xp_type =
-#ifdef RPC_VSOCK
-		(flags & SVC_VC_CREATE_VSOCK) ? XPRT_VSOCK_RENDEZVOUS :
-#endif /* VSOCK */
-		XPRT_TCP_RENDEZVOUS;
+	xprt->xp_type = XPRT_TCP_RENDEZVOUS;
 
 	if (ops.xp_recv == NULL) {
 		ops.xp_recv = rendezvous_request;
 		ops.xp_stat = rendezvous_stat;
 		/* XXX wow */
 		ops.xp_getargs = (bool(*)
-				  (SVCXPRT *, struct svc_req *, xdrproc_t,
+				  (struct svc_req *, xdrproc_t,
 				   void *, void *))abort;
 		ops.xp_reply = (bool(*)
-				(SVCXPRT *, struct svc_req *req,
-				 struct rpc_msg *))abort;
+				(struct svc_req *req))abort;
 		ops.xp_freeargs = (bool(*)
-				   (SVCXPRT *, struct svc_req *, xdrproc_t,
+				   (struct svc_req *, xdrproc_t,
 				    void *))abort;
-		ops.xp_destroy = svc_rdvs_destroy;
+		ops.xp_destroy = svc_vc_destroy_it;
 		ops.xp_control = svc_vc_rendezvous_control;
-		ops.xp_lock = svc_vc_lock;
-		ops.xp_unlock = svc_vc_unlock;
 		ops.xp_getreq = svc_getreq_default;
 		ops.xp_dispatch = svc_dispatch_default;
 		ops.xp_recv_user_data = NULL;	/* no default */
@@ -1181,21 +1041,15 @@ svc_clean_idle2_func(SVCXPRT *xprt, void *arg)
 	if (xprt->xp_ops == NULL)
 		goto unlock;
 
-	if (xprt->xp_flags & SVC_XPRT_FLAG_DESTROYED) {
-		/* XXX should not happen--but do no harm */
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"%s: destroyed xprt %p seen in clean idle\n",
-			__func__,
-			xprt);
-		goto unlock;
-	}
-
 	if (xprt->xp_ops->xp_recv != svc_vc_recv)
 		goto unlock;
 
+	if (xprt->xp_flags & (SVC_XPRT_FLAG_DESTROYED | SVC_XPRT_FLAG_UREG))
+		goto unlock;
+
 	{
 		/* XXX nb., safe because xprt type is verfied */
-		struct x_vc_data *xd = xprt->xp_p1;
+		struct svc_vc_xprt *xd = VC_DR(REC_XPRT(xprt));
 		if (!xd->shared.nonblock)
 			goto unlock;
 
@@ -1243,8 +1097,10 @@ __svc_clean_idle2(int timeout, bool cleanblock)
 
 	++active;
 
+#ifdef _HAVE_GSSAPI
 	/* trim gss context cache */
 	authgss_ctx_gc_idle();
+#endif /* _HAVE_GSSAPI */
 
 	/* trim xprts (not sorted, not aggressive [but self limiting]) */
 	memset(&acc, 0, sizeof(struct svc_clean_idle_arg));
@@ -1267,131 +1123,3 @@ __svc_clean_idle2(int timeout, bool cleanblock)
 	return (rslt);
 
 }				/* __svc_clean_idle2 */
-
-/*
- * Create an RPC client handle from an active service transport
- * handle, i.e., to issue calls on the channel.
- *
- * If flags & SVC_VC_CLNT_CREATE_DEDICATED, the supplied xprt will be
- * unregistered and disposed inline.
- */
-CLIENT *
-clnt_vc_ncreate_svc(SVCXPRT *xprt, const rpcprog_t prog,
-		    const rpcvers_t vers, const uint32_t flags)
-{
-	struct x_vc_data *xd;
-	CLIENT *clnt;
-
-	mutex_lock(&xprt->xp_lock);
-
-	xd = (struct x_vc_data *)xprt->xp_p1;
-
-	/* XXX return allocated client structure, or allocate one if none
-	 * is currently allocated */
-
-	clnt =
-	    clnt_vc_ncreate2(xprt->xp_fd, &xprt->xp_remote.nb, prog, vers,
-			     xd->shared.sendsz, xd->shared.recvsz,
-			     CLNT_CREATE_FLAG_SVCXPRT);
-
-	mutex_unlock(&xprt->xp_lock);
-
-	/* for a dedicated channel, unregister and free xprt */
-	if ((flags & SVC_VC_CREATE_ONEWAY) && (flags & SVC_VC_CREATE_DISPOSE)) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"%s:  disposing--calls svc_vc_destroy\n", __func__);
-		svc_vc_destroy(xprt, 0, __func__, __LINE__);
-	}
-
-	return (clnt);
-}
-
-/*
- * Create an RPC SVCXPRT handle from an active client transport
- * handle, i.e., to service RPC requests.
- *
- * If flags & SVC_VC_CREATE_CL_FLAG_DEDICATED, then clnt is also
- * deallocated without closing cl->cl_p1->ct_fd.
- */
-SVCXPRT *
-svc_vc_ncreate_clnt(CLIENT *clnt, const u_int sendsz,
-		    const u_int recvsz, const uint32_t flags)
-{
-	int fd;
-	socklen_t len;
-	struct x_vc_data *xd = (struct x_vc_data *)clnt->cl_p1;
-	struct ct_data *ct = &xd->cx.data;
-	struct sockaddr_storage addr;
-	struct __rpc_sockinfo si;
-	SVCXPRT *xprt = NULL;
-	bool xprt_allocd;
-
-	fd = ct->ct_fd;
-	rpc_dplx_rlc(clnt);
-	rpc_dplx_slc(clnt);
-
-	len = sizeof(struct sockaddr_storage);
-	if (getpeername(fd, (struct sockaddr *)(void *)&addr, &len) < 0) {
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"%s: could not retrieve remote addr", __func__);
-		goto unlock;
-	}
-
-	/*
-	 * make a new transport
-	 */
-
-	xprt = makefd_xprt(fd, sendsz, recvsz, &xprt_allocd);
-	if ((!xprt) || (!xprt_allocd))	/* ref'd existing xprt handle */
-		goto unlock;
-
-	__rpc_set_address(&xprt->xp_remote, &addr, len);
-
-	if (__rpc_fd2sockinfo(fd, &si) && si.si_proto == IPPROTO_TCP) {
-		len = 1;
-		(void) setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &len,
-				  sizeof(len));
-	}
-
-	xd->sx.maxrec = __svc_maxrec;	/* XXX check */
-
-#if 0				/* XXX wont currently support */
-	if (xd->sx.maxrec != 0) {
-		fflags = fcntl(fd, F_GETFL, 0);
-		if (fflags == -1)
-			return (FALSE);
-		if (fcntl(fd, F_SETFL, fflags | O_NONBLOCK) == -1)
-			return (FALSE);
-		if (xd->shared.recvsz > xd->sx.maxrec)
-			xd->shared.recvsz = xd->sx.maxrec;
-		cd->nonblock = TRUE;
-		__xdrrec_setnonblock(&cd->xdrs, xd->sx.maxrec);
-	} else
-		xd->shared.nonblock = FALSE;
-#else
-	xd->shared.nonblock = FALSE;
-#endif
-	(void)clock_gettime(CLOCK_MONOTONIC_FAST, &xd->sx.last_recv);
-
-	/* conditional xprt_register */
-	if ((!(__svc_params->flags & SVC_FLAG_NOREG_XPRTS))
-	    && (!(flags & SVC_VC_CREATE_XPRT_NOREG)))
-		xprt_register(xprt);
-
-	/* If creating a dedicated channel collect the supplied client
-	 * without closing fd */
-	if ((flags & SVC_VC_CREATE_ONEWAY) && (flags & SVC_VC_CREATE_DISPOSE)) {
-		ct->ct_closeit = FALSE;	/* must not close */
-		rpc_dplx_ruc(clnt);
-		rpc_dplx_suc(clnt);
-		CLNT_DESTROY(clnt);	/* clean up immediately */
-		goto out;
-	}
-
-unlock:
-	rpc_dplx_ruc(clnt);
-	rpc_dplx_suc(clnt);
-
-out:
-	return (xprt);
-}
